import _ from "lodash-es";
import { Buffer } from 'buffer'

export enum FOOT {
  F2 = 0,
  F5 = 1,
}
export enum SPEED {
  SPEED1DIV5 = 15,
  SPEED2 = 20,
  SPEED3 = 30,
  SPEED4 = 40,
  SPEED5 = 50,
  SPEED6 = 60,
  SPEED7 = 70,
  SPEED8 = 80,
  SPEED9 = 90,
  SPEED10 = 100,
  SPEED11 = 110,
  SPEED12 = 120,
}

export enum DENSITY {
  DNESITY0 = 0,
  DNESITY1 = 1,
  DNESITY2 = 2,
  DNESITY3 = 3,
  DNESITY4 = 4,
  DNESITY5 = 5,
  DNESITY6 = 6,
  DNESITY7 = 7,
  DNESITY8 = 8,
  DNESITY9 = 9,
  DNESITY10 = 10,
  DNESITY11 = 11,
  DNESITY12 = 12,
  DNESITY13 = 13,
  DNESITY14 = 14,
  DNESITY15 = 15,
}

export enum DIRECTION {
  FORWARD = 0,
  BACKWARD = 1,
}

export enum MIRROR {
  NORMAL = 0,
  MIRROR = 1,
}
export enum CODEPAGE {
  PC437 = 437,
  PC850 = 850,
  PC852 = 852,
  PC860 = 860,
  PC863 = 863,
  PC865 = 865,
  WPC1250 = 1250,
  WPC1252 = 1252,
  WPC1253 = 1253,
  WPC1254 = 1254,
}
export enum FONTTYPE {
  FONT_0 = "0.TTF",
  Bold = "1.TTF",
  SongTi = "2.TTF",
  Thai = "4.TTF",
  FONT_1 = "1",
  FONT_2 = "2",
  FONT_3 = "3",
  FONT_4 = "4",
  FONT_5 = "5",
  FONT_6 = "6",
  FONT_7 = "7",
  FONT_8 = "8",
  FONT_9 = "9",
  FONT_10 = "10",
  SIMPLIFIED_16_CHINESE = "TSS16.BF2",
  SIMPLIFIED_20_CHINESE = "TSS20.BF2",
  SIMPLIFIED_24_CHINESE = "TSS24.BF2",
  SIMPLIFIED__32_CHINESE = "TSS32.BF2",
  TRADITIONAL_CHINESE = "TST24.BF2",
  KOREAN = "K",
}
export enum ROTATION {
  ROTATION_0 = 0,
  ROTATION_90 = 90,
  ROTATION_180 = 180,
  ROTATION_270 = 270,
}
export enum FONTMUL {
  MUL_1 = 1,
  MUL_2 = 2,
  MUL_3 = 3,
  MUL_4 = 4,
  MUL_5 = 5,
  MUL_6 = 6,
  MUL_7 = 7,
  MUL_8 = 8,
  MUL_9 = 9,
  MUL_10 = 10,
}
export enum BITMAP_MODE {
  OVERWRITE = 0,
  OR = 1,
  XOR = 2,
}
export enum READABEL {
  DISABLE = 0,
  EANBEL = 1,
}
export enum Shape {
  Square = "0",
  Rectangle = "1",
}
export enum EEC {
  LEVEL_L = "L",
  LEVEL_M = "M",
  LEVEL_Q = "Q",
  LEVEL_H = "H",
}
export enum BARCODETYPE {
  CODE128 = "128",
  CODE128M = "128M",
  EAN128 = "EAN128",
  ITF25 = "25",
  ITF25C = "25C",
  CODE39 = "39",
  CODE39C = "39C",
  CODE39S = "39S",
  CODE93 = "93",
  EAN13 = "EAN13",
  EAN13_2 = "EAN13+2",
  EAN13_5 = "EAN13+5",
  EAN8 = "EAN8",
  EAN8_2 = "EAN8+2",
  EAN8_5 = "EAN8+5",
  CODABAR = "CODA",
  POST = "POST",
  UPCA = "UPCA",
  UPCA_2 = "UPCA+2",
  UPCA_5 = "UPCA+5",
  UPCE = "UPCE13",
  UPCE_2 = "UPCE13+2",
  UPCE_5 = "UPCE13+5",
  CPOST = "CPOST",
  MSI = "MSI",
  MSIC = "MSIC",
  PLESSEY = "PLESSEY",
  ITF14 = "ITF14",
  EAN14 = "EAN14",
}
export enum RESPONSE_MODE {
  ON = "ON",
  OFF = "OFF",
  BATCH = "BATCH",
}
export enum ENABLE {
  ON = "ON",
  OFF = "OFF",
}

export class LabelCommand {
  command: number[] = [];
  constructor(width: number, height: number, gap: number) {
    this.command = new Array();
    this.addSize(width, height);
    this.addGap(gap);
  }
  public clrCommand() {
    this.command = [];
  }
  addStrToCommand(str: string): void {
    if (str !== "") {
      const bs = Buffer.from(str, "utf8");
      for (let i = 0; i < bs.length; ++i) {
        this.command.push(bs[i]);
      }
    }
  }
  /**
   * 追加编码
   * @param number 
   */
  public addToCommand(number: number[]) {
    this.command = this.command.concat(number);
  }
  /**
   * 定义两张标签之间的间隙距离
   * @param gap 定义标签间隙高度 (mm)
   */
  addGap(gap: number) {
    let str = "GAP " + gap + " mm," + 0 + " mm\r\n";
    this.addStrToCommand(str);
  }
  /**
   * 设定黑标高度及使用者定义标签印完后标签额外送出的长度
   * 连续纸模式 GAP 0,0
   * @param bline 额外送出纸张长度 (mm)
   */
  addBline(bline: number) {
    let str = "BLINE " + bline + " mm," + 0 + " mm\r\n";
    this.addStrToCommand(str);
  }
  /**
   * 定义标签纸的宽度及高度
   * @param height 定义标签纸的高度 (mm)
   * @param width 定义标签纸的宽度 (mm)
   */
  addSize(width: number, height: number): void {
    const str = "SIZE " + width + " mm," + height + " mm\r\n";
    this.addStrToCommand(str);
  }
  addCashdrwer(m: FOOT, t1: number, t2: number): void {
    let str: string = `CASHDRAWER ${m},${t1},${t2}\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 定义标签于打印完后额外推出的长度，特別是在使用自动剥纸器或裁刀功能时，可用来
   * 调校标签停止的位置，在打印下一张时打印机会将原先多推出或少推出的部分以回拉方
   * 式补偿回来。此方式适用于剥离模式和裁刀模式。
   * @param offset 偏移距离 (mm)
   */
  addOffset(offset: number): void {
    let str: string = `OFFSET ${offset} mm\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 设定打印机的打印速度
   * @param speed
   */
  addSpeed(speed: SPEED): void {
    let str: string = `SPEED ${speed / 10}\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 设定打印机的打印浓度
   * @param density
   */
  addDensity(density: DENSITY): void {
    let str: string = `DENSITY ${density}\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 设定打印方向，此设定将被记录于 EEPROM
   * @param direction  0-翻转 1-顺序
   * @param mirror
   */
  addDirection(direction: DIRECTION, mirror: MIRROR): void {
    let str: string = `DIRECTION ${direction}, ${mirror}\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 定义标签纸上的相對于原点的参考点座标
   * @param x 水平座标，单位 “dot”
   * @param y 垂直座标，单位 “dot”
   */
  addReference(x: number, y: number): void {
    let str: string = `REFERENCE ${x}, ${y}\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 该指令表示标签打印偏移量多少设置
   * @param shift 最大值为 1 inch，以点（dot）为单位
   */
  addShift(shift: number): void {
    let str: string = `SHIFT ${shift}\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 清除数据缓存
   * 注意事项：此项指令必须置于 SIZE 指令之后
   */
  addCls(): void {
    let str: string = `CLS\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 将标签纸向前推送指定的长度 (送出纸)
   * @param dot 单位：dot
   */
  addFeed(dot: number): void {
    let str: string = `FEED ${dot}\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 将标签纸向后回拉指定的长度 (回收纸)
   * @param dot  单位：dot
   */
  addBackFeed(dot: number): void {
    let str: string = `BACKFEED ${dot}\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 将标签向前推送至下一张标签的起始位置
   */
  addFormFeed(): void {
    let str: string = `FORMFEED\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 在使用含有间隙或黑标的标签纸时，若不能确定第一张标签纸是否在正确打印位置时，
   * 此指令可将标签纸向前推送至下一张标签纸的起点开始打印
   */
  addHome(): void {
    let str: string = `HOME\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 将存于数据缓存的标签打印
   * @param m 打印张数
   * @param n 每张标签需重复打印的张数
   */
  addPrint(m: number, n?: number): void {
    let str: string = n ? `PRINT ${m}, ${n}\r\n` : `PRINT ${m}\r\n`;
    this.addStrToCommand(str);
  }
  addCodePage(page: CODEPAGE): void {
    let str: string = `CODEPAGE ${page}\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 控制打印机蜂鳴器发聲的頻率，共有 10 阶，并可由 “interval” 参数设定音与音的间隔时间
   * @param level 音阶：0~9
   * @param interval 间隔时间: 1~4095
   */
  addSound(level: number, interval: number): void {
    let str: string = `SOUND ${level}, ${interval}\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 限定定间隙校正执行的最大长度，若在此长度范围内无法测得间隙存在，则将感应器模式定在连续纸模式下
   * @param n 间隙校正的最大长度
   */
  addLimitFeed(n: number): void {
    let str: string = `LIMITFEED ${n}\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 功能：不经自测动作，直接将打印机讯息印在标签纸上。
   */
  addSelfTest(): void {
    let str: string = `SELFTEST\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 画线或长条物
   * @param x 线条左上角 X 座标，单位 dot
   * @param y 线条左上角 Y 座标，单位 dot
   * @param width 线条宽度，单位 dot
   * @param height 线条高度，单位 dot
   */
  addBar(x: number, y: number, width: number, height: number): void {
    let str: string = `BAR ${x}, ${y}, ${width}, ${height}\r\n`;
    this.addStrToCommand(str);
  }
  /***
   * 添加文本
   */
  addText(
    x: number,
    y: number,
    font: FONTTYPE,
    rotation: ROTATION,
    Xscal: FONTMUL,
    Yscal: FONTMUL,
    text: string
  ): void {
    const str =
      "TEXT " +
      x +
      "," +
      y +
      ',"' +
      font +
      '",' +
      rotation +
      "," +
      Xscal +
      "," +
      Yscal +
      ',"' +
      text +
      '"\r\n';
    this.addStrToCommand(str);
  }
  /**
   * 添加自定义指令
   * @param command 指令字符串
   */
  addUserCommand(command: string): void {
    this.addStrToCommand(command);
  }
  addCOUNTER(number: number, step: number): void {
    let str: string = `SET COUNTER @${number} ${step}\r\n`;
    this.addStrToCommand(str);
  }
  addQueryPrinterStatus(mode: RESPONSE_MODE): void {
    let str: string = `SET RESPONSE ${mode}\r\n`;
    this.addStrToCommand(str);
  }
  addPartialCutter(enable: ENABLE): void {
    let str: string = `SET PARTIAL_CUTTER ${enable}\r\n`;
    this.addStrToCommand(str);
  }
  addPrintKey(m: number | ENABLE) {
    if (typeof m == "number") {
      let str: string = "SET PRINTKEY " + m + "\r\n";
      this.addStrToCommand(str);
    } else {
      let str: string = "SET PRINTKEY " + m + "\r\n";
      this.addStrToCommand(str);
    }
  }
  addCutterBatch() {
    const str = "SET CUTTER BATCH\r\n";
    this.addStrToCommand(str);
  }
  addCutterPieces(number: number) {
    const str = "SET CUTTER " + number + "\r\n";
    this.addStrToCommand(str);
  }
  addQueryPrinterCodePage() {
    const str = "~!I\r\n";
    this.addStrToCommand(str);
  }
  addQueryPrinterFile() {
    const str = "~!F\r\n";
    this.addStrToCommand(str);
  }
  addQueryPrinterMemory() {
    const str = "~!A\r\n";
    this.addStrToCommand(str);
  }
  addQueryPrinterLife() {
    const str = "~!@\r\n";
    this.addStrToCommand(str);
  }
  addResetPrinter(): void {
    this.command.push(27, 33, 82);
  }
  addReprint(enable: RESPONSE_MODE): void {
    let str: string = `SET REPRINT ${enable}\r\n`;
    this.addStrToCommand(str);
  }
  addHead(enable: ENABLE): void {
    let str: string = `SET HEAD ${enable}\r\n`;
    this.addStrToCommand(str);
  }

  addRibbon(enable: ENABLE): void {
    let str: string = `SET RIBBON ${enable}\r\n`;
    this.addStrToCommand(str);
  }

  addKeyOne(enable: ENABLE): void {
    let str: string = `SET KEY1 ${enable}\r\n`;
    this.addStrToCommand(str);
  }

  addKeyTwo(enable: ENABLE): void {
    let str: string = `SET KEY2 ${enable}\r\n`;
    this.addStrToCommand(str);
  }
  addPeel(enable: RESPONSE_MODE): void {
    let str: string = `SET PEEL ${enable}\r\n`;
    this.addStrToCommand(str);
  }

  addTear(mode: RESPONSE_MODE): void {
    let str: string = `SET TEAR ${mode}\r\n`;
    this.addStrToCommand(str);
  }

  addCutter(enable: RESPONSE_MODE): void {
    let str: string = `SET CUTTER ${enable}\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 获取命令
   */
  getCommand() {
    return this.command;
  }
  /**
   * 获取命令字符串
   */
  getCommandStr(command: number[]) {
    // 创建一个Buffer实例来存储字节数组
    const buffer = Buffer.from(command);

    const str = buffer.toString('utf8'); // 指定utf8编码

    return str;
  }
  addQueryPrinterType(): void {
    let str: string = `~!T\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 绘制 QRCODE 二维条码
   * @param x QRCODE 条码左上角 X 座标
   * @param y QRCODE 条码左上角 Y 座标
   * @param level 错误纠正能力等级
   * @param cellwidth 1~10
   * @param rotation 顺时针旋转角度
   * @param data 条码资料内容
   */
  public addQRCode(
    x: number,
    y: number,
    level: EEC,
    cellwidth: number,
    rotation: ROTATION,
    data: string
  ): void {
    let str: string = `QRCODE ${x}, ${y}, ${level}, ${cellwidth}, A, ${rotation}, "${data}"\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 将图像缓存中指定的区域做反相的动作
   * @param x 欲清除区域的左上角 X 座标，单位：dot
   * @param y 欲清除区域的左上角 Y 座标，单位：dot
   * @param xwidth 欲清除区域的宽度，单位：dot
   * @param yheight 欲清除区域的高度，单位：dot
   */
  addReverse(x: number, y: number, xwidth: number, yheight: number): void {
    let str = `REVERSE ${x}, ${y}, ${xwidth}, ${yheight}\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 清除影像缓冲区中指定大小的区域
   * @param x 欲清除区域的左上角 X 座标，单位：dot
   * @param y 欲清除区域的左上角 Y 座标，单位：dot
   * @param xwidth 欲清除区域的宽度，单位：dot
   * @param yheight 欲清除区域的高度，单位：dot
   */
  addErase(x: number, y: number, xwidth: number, yheight: number): void {
    let str = `ERASE ${x},${y},${xwidth},${yheight}\r\n`;
    this.addStrToCommand(str);
  }
  addDMATRIX(
    x: number,
    y: number,
    width: number,
    height: number,
    content: string
  ): void {
    let str = `DMATRIX ${x},${y},${width},${height},"${content}"\r\n`;
    this.addStrToCommand(str);
  }
  /**
   * 印制一维条码，
   * @param x 明定条码左上角 X 座标
   * @param y 明定条码左上角 Y 座标
   * @param type
   * @param height
   * @param readable
   * @param rotation
   * @param narrow
   * @param width
   * @param content
   */
  add1DBarcode(
    x: number,
    y: number,
    type: BARCODETYPE,
    height: number,
    readable: READABEL,
    rotation: ROTATION,
    narrow: number,
    width: number,
    content: string
  ): void {
    let str = `BARCODE ${x},${y},"${type}",${height},${readable},${rotation},${narrow},${width},"${content}"\r\n`;
    this.addStrToCommand(str);
  }

  /**
   * 绘制方框
   * @param x 方框左上角 X 座标，单位：dot
   * @param y 方框左上角 Y 座标，单位：dot
   * @param xend 方框右下角 X 座标，单位：dot
   * @param yend 方框右下角 X 座标，单位：dot
   * @param thickness 方框线条粗細，单位：dot
   */
  addBox(
    x: number,
    y: number,
    xend: number,
    yend: number,
    thickness: number
  ): void {
    let str = `BOX ${x},${y},${xend},${yend},${thickness}\r\n`;
    this.addStrToCommand(str);
  }
}
